/* The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Eric Hodel's <drbrain@segment7.net> code.
 * 
 * The Initial Developer of the Original Code is Eric Hodel.
 * Portions created by Eric Hodel are Copyright (C) 2001 Eric
 * Hodel. All Rights Reserved.
 * 
 * Other Contributors:
 *      Christopher Hoess <choess@force.stwing.upenn.edu>
 *      Tim Taylor <tim@tool-man.org>
 */


/** 
 * LinkToolbarHandler is a Singleton that displays LINK elements 
 * and nodeLists of LINK elements in the Link Toolbar.  It
 * associates the LINK with a corresponding LinkToolbarItem based 
 * on the LINK's REL attribute.  LinkToolbarHandler is also a 
 * Factory and will create LinkToolbarItems as necessary.
 */
function LinkToolbarHandler() {
  /*
   * XXX: refactor factory methods into a seperate 
   * LinkToolbarItemFactory class
   */

  this.items = new Array();

  this.handleLinks = function(nodeList) {
    // FIXME: test nodeList for null

    for (var i = 0; i < nodeList.length; i++) {
      if (nodeList.item(i).rel || nodeList.item(i).rev) 
        linkToolbarHandler.handle(nodeList.item(i));
    }
    ddump("LinkToolbarHandler::handleLinks: handled " + nodeList.length + " links");
  }

  this.handle = function(linkElement) {
    convertRevMade(linkElement);

    if (isExcludedLink(linkElement)) return;

    var relAttributes = linkElement.rel.split(" ");
    for (var i = 0; i < relAttributes.length; i++) {
      var linkType = getLinkType(relAttributes[i]);
      this.getItemForLinkType(linkType).addLink(linkElement);
    }
  }

  this.getItemForLinkType = function(linkType) {
    if (!this.items[linkType])
      this.items[linkType] = createItemForLinkType(linkType);

    return this.items[linkType];
  }

  function createItemForLinkType(linkType) {
    ddump("LinkToolbarHandler::createItemForLinkType: type = " + linkType);

    if (!document.getElementById("link-" + linkType))
      return new LinkToolbarTransientMenu(linkType);

    // XXX: replace switch with polymorphism
    switch(document.getElementById("link-" + linkType).tagName) {
      case "button":
        return new LinkToolbarButton(linkType);

      case "menuitem":
        return new LinkToolbarMenuItem(linkType);

      case "menu":
        return new LinkToolbarMenu(linkType);

      default:
        return new LinkToolbarTransientMenu(linkType);
    }
  }

  this.clearAllItems = function() {
    for (var linkType in this.items)
      this.items[linkType].clear();
  }

  function convertRevMade(linkElement) {
    if (!linkElement.rel && linkElement.rev) {
      if (linkElement.rev.toLowerCase() == "made")
        linkElement.rel = linkElement.rev;
    }
  }

  function isExcludedLink(linkElement) {
    if (!linkElement.rel) return true;
    if (isStylesheetLink(linkElement)) return true;

    return false;
  }

  function isStylesheetLink(linkElement) {
    if (linkElement.rel.toLowerCase().search("stylesheet") != -1)
      return true;
    else
      return false;
  }

  function getLinkType(relAttribute) {
    switch (relAttribute.toLowerCase()) {
      case "home":
      case "start":
      case "top":
      case "origin":
        return "top";

      case "up":
      case "parent":
        return "up";

      case "begin":
      case "first":
        return "first";

      case "next":
      case "child":
        return "next";

      case "prev":
      case "previous":
        return "prev";

      case "end":
      case "last":
        return "last";

      case "author":
      case "made":
        return "author";

      case "contents":
      case "toc":
        return "toc";

      default:
        return relAttribute.toLowerCase();
    }
  }
}
const linkToolbarHandler = new LinkToolbarHandler();
